@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWObject.h 202 2008-10-12 04:02:18Z vadim $
+  $Id: FWObject.h 236 2008-12-25 20:36:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -40,6 +40,7 @@
 
 #include <fwbuilder/libfwbuilder-config.h>
 #include <fwbuilder/FWException.h>
+#include <fwbuilder/ObjectMatcher.h>
 
 namespace libfwbuilder
 {
@@ -91,7 +92,7 @@ private:
      *
      * dbroot is assigned by method FWObjectDatabase::create 
      */
-    FWObject *dbroot;
+    FWObjectDatabase *dbroot;
     int id;
     bool ro;
     std::string name;
@@ -144,7 +145,7 @@ protected:
      *  Parameter 'root' is a pointer at the root object of the tree,
      *  we are going to add this newly created object to.
      */
-    FWObject(const FWObject *root,bool prepopulate);
+    FWObject(const FWObjectDatabase *root, bool prepopulate);
 
 public:
 
@@ -153,7 +154,7 @@ public:
     class tree_iterator {
         friend class libfwbuilder::FWObject;
 
-        FWObject                       *node;
+        FWObject *node;
 
       public:
         tree_iterator()                        { node=NULL;    }
@@ -215,7 +216,8 @@ public:
      * ones are issued. All children of 'this' are destroyed and new ones
      * are created recursively as copies of corresponding children of obj.
      */
-    virtual FWObject& duplicate(const FWObject *obj, bool preserve_id = true) throw(FWException);
+    virtual FWObject& duplicate(const FWObject *obj,
+                                bool preserve_id = true) throw(FWException);
     
     /**
      * This method works just like  duplicate, except it does not destroy
@@ -282,7 +284,8 @@ public:
     /**
      * Sets pointer to the database root
      */
-    void setRoot(const FWObject *_dbroot) { dbroot = (FWObject*)_dbroot; }
+    void setRoot(const FWObjectDatabase *_dbroot)
+    { dbroot = (FWObjectDatabase*)_dbroot; }
 
     /**
      *   Returns a string that represents a path to the object
@@ -351,7 +354,7 @@ public:
     /*
      * verify whether given object type is approppriate as a child
      */
-    virtual bool  validateChild(FWObject *o);
+    virtual bool validateChild(FWObject *o);
 
     /**
      * forcefully destroys all objects in the subtree under this,
@@ -379,22 +382,30 @@ public:
      * and itslef. If it is true, method proceeds to all chidren of all
      * objects starting from current one.
      */
-    virtual FWObject* getById  (int id, bool recursive=false);
+    virtual FWObject* getById(int id, bool recursive=false);
 
 
     /**
-     * Returns list of direct children of current object
-     * whose getTypeName() same as given.
-     * If not found, empty list is returned.
+     * Returns list of direct children of current object whose
+     * getTypeName() is equal type_name.  If not found, empty list is
+     * returned.
      */
     virtual std::list<FWObject*> getByType(const std::string &type_name) const;
 
+    /**
+     * Returns list of children of current object whose getTypeName()
+     * is equal type_name.  If not found, empty list is
+     * returned. Search recursively including all direct children of this.
+     */
+    virtual std::list<FWObject*> getByTypeDeep(const std::string &type_name) const;
+
     /**
      * Returns list of direct children of current object
      * whose getTypeName() same as given.
      * If not found, end() is returned.
      */
-    virtual FWObjectTypedChildIterator findByType(const std::string &type_name) const;
+    virtual FWObjectTypedChildIterator findByType(
+        const std::string &type_name) const;
 
     /**
      * Returns first of direct children of current object
@@ -428,8 +439,8 @@ public:
     void findDependencies(std::list<FWObject*> &deps);
 
     
-    virtual void    setDirty(bool f);
-    virtual bool    isDirty();
+    virtual void setDirty(bool f);
+    virtual bool isDirty();
 
     /**
      * sets read-only flag for the object. FWObject checks all objects
@@ -437,13 +448,36 @@ public:
      * is read-only. Marking an object read-only automatically makes
      * the whole tree branch under it read-only.
      */
-    virtual void    setReadOnly(bool f);
-    virtual bool    isReadOnly();
-    virtual void    checkReadOnly() throw(FWException);
+    virtual void setReadOnly(bool f);
+    virtual bool isReadOnly();
+    virtual void checkReadOnly() throw(FWException);
+
+    /**
+     * return true if this object can be copied around and put in the
+     * tree in one of the system folders. False if it must be a child
+     * of some other primary obejct. The latter category includes
+     * physAddress, interface, IPv4 and IPv6 if they are children of
+     * interface, rulesets, rule elements and rules.
+     */
+    virtual bool isPrimaryObject() const;
+
+    /**
+     * virtual dispatch method for the double dispatch
+     * pattern. Compiler::complexMatch() calls this virt. method,
+     * which in turn calls Compiler::checkComplexMatch where first
+     * argument has specific type. Only classes for which we implement
+     * special algorithm in corresponding checkComplexMatch() have
+     * virtual dispatchComplexMatch reimplemented. For other classes
+     * this base method is called (which always returns false).
+     */
+    virtual bool dispatchComplexMatch(ObjectMatcher*, FWObject*)
+    { return false; }
 
     // Attributes iterator
-    std::map<std::string, std::string>::const_iterator dataBegin() { return data.begin(); }
-    std::map<std::string, std::string>::const_iterator dataEnd  () { return data.end();   }
+    std::map<std::string, std::string>::const_iterator dataBegin()
+    { return data.begin(); }
+    std::map<std::string, std::string>::const_iterator dataEnd()
+    { return data.end();   }
 
 };
 
