@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ICMP6Service.h 123 2008-07-05 17:38:52Z vadim $
+  $Id: ICMP6Service.h 237 2008-12-28 07:12:17Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -41,13 +41,16 @@ private:
 public:
 
     ICMP6Service();
-    ICMP6Service(const FWObject *root,bool prepopulate);
+    ICMP6Service(const FWObjectDatabase *root,bool prepopulate);
     virtual ~ICMP6Service();
 
     DECLARE_FWOBJECT_SUBTYPE(ICMP6Service);
 
     virtual std::string getProtocolName();
     virtual int getProtocolNumber();
+
+    virtual bool isV4Only() { return false; }
+    virtual bool isV6Only() { return true; }
 };
 
 }
